package com.gree.mall.manager.logic.quality;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.quality.QualityFeedbackBean;
import com.gree.mall.manager.bean.quality.QualityFeedbackCountBean;
import com.gree.mall.manager.bean.quality.QualityFeedbackVO;
import com.gree.mall.manager.commonmapper.CommonMapper;
import com.gree.mall.manager.enums.quality.ExamineQualityFeedbackStatusEnum;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.QualityFeedback;
import com.gree.mall.manager.plus.entity.QualityFeedbackItem;
import com.gree.mall.manager.plus.service.QualityFeedbackItemService;
import com.gree.mall.manager.plus.service.QualityFeedbackService;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class QualityFeedbackLogic {

    private final CommonLogic commonLogic;
    private final CommonMapper commonMapper;
    private final QualityFeedbackService qualityFeedbackService;
    private final QualityFeedbackItemService qualityFeedbackItemService;

    public IPage<QualityFeedbackVO> list(ZfireParamBean zfireParamBean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        FieldUtils.supplyParam(zfireParamBean, QualityFeedbackVO.class, adminUser);

        return commonMapper.qualityFeedbackPage(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
    }

    public QualityFeedbackBean detail(String id) {
        QualityFeedbackBean bean = new QualityFeedbackBean();
        QualityFeedback qualityFeedback = qualityFeedbackService.getById(id);
        BeanUtils.copyProperties(qualityFeedback, bean);

        final List<QualityFeedbackItem> items = qualityFeedbackItemService.lambdaQuery()
                .eq(QualityFeedbackItem::getQualityFeedbckId, id)
                .list();
        bean.setItems(items);

        return bean;
    }

    public QualityFeedbackCountBean count() {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        QualityFeedbackCountBean bean = new QualityFeedbackCountBean();
        Integer saveCount = qualityFeedbackService.lambdaQuery()
                .eq(QualityFeedback::getStatus, ExamineQualityFeedbackStatusEnum.SAVE.getKey())
                .in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()), QualityFeedback::getWebsitId, adminUser.getAdminWebsitIds())
                .count();
        Integer waitCount = qualityFeedbackService.lambdaQuery()
                .eq(QualityFeedback::getStatus, ExamineQualityFeedbackStatusEnum.WAIT.getKey())
                .in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()), QualityFeedback::getWebsitId, adminUser.getAdminWebsitIds())
                .count();
        Integer rejectCount = qualityFeedbackService.lambdaQuery()
                .eq(QualityFeedback::getStatus, ExamineQualityFeedbackStatusEnum.REJECT.getKey())
                .in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()), QualityFeedback::getWebsitId, adminUser.getAdminWebsitIds())
                .count();
        bean.setSaveCount(saveCount);
        bean.setWaitCount(waitCount);
        bean.setRejectCount(rejectCount);
        return bean;
    }

    public void add(QualityFeedbackCountBean bean) {
    }

    public void update(QualityFeedbackCountBean bean) {
    }

    public void confirm(String id, ExamineQualityFeedbackStatusEnum status) {
    }
}
